import { AuthProvider, GetListParams, GetOneParams, GetManyParams, GetManyReferenceParams, CreateParams, UpdateParams, UpdateManyParams, DeleteParams, DeleteManyParams, DataProvider } from 'react-admin';
import { CreateAxiosDefaults, AxiosInterceptorOptions } from 'axios';
import { PrismaClient } from '@prisma/client';
import * as ra_core from 'ra-core';

type AuditOptions = {
    model: {
        create: Function;
    };
    authProvider: AuthProvider;
    columns?: {
        id?: string;
        date?: string;
        resource?: string;
        action?: string;
        payload?: string;
        author?: string;
    };
    enabledForAction?: {
        create?: boolean;
        update?: boolean;
        delete?: boolean;
    };
    enabledResources?: string[];
};
declare const defaultAuditOptions: Pick<AuditOptions, "columns" | "enabledForAction">;
type AuditActions = "create" | "update" | "delete";

type RaPayload<T = string> = GetListRequest<T> | GetOneRequest<T> | GetManyRequest<T> | GetManyReferenceRequest<T> | CreateRequest<T> | UpdateRequest<T> | UpdateManyRequest<T> | DeleteRequest<T> | DeleteManyRequest<T>;
type GetListRequest<T = string> = {
    method: "getList";
    params: GetListParams;
    resource: T;
    model?: string;
};
type GetOneRequest<T = string> = {
    method: "getOne";
    params: GetOneParams;
    resource: T;
    model?: string;
};
type GetManyRequest<T = string> = {
    method: "getMany";
    params: GetManyParams;
    resource: T;
    model?: string;
};
type GetManyReferenceRequest<T = string> = {
    method: "getManyReference";
    params: GetManyReferenceParams;
    resource: T;
    model?: string;
};
type CreateRequest<T = string> = {
    method: "create";
    params: CreateParams;
    resource: T;
    model?: string;
};
type UpdateRequest<T = string> = {
    method: "update";
    params: UpdateParams;
    resource: T;
    model?: string;
};
type UpdateManyRequest<T = string> = {
    method: "updateMany";
    params: UpdateManyParams;
    resource: T;
    model?: string;
};
type DeleteRequest<T = string> = {
    method: "delete";
    params: DeleteParams;
    resource: T;
    model?: string;
};
type DeleteManyRequest<T = string> = {
    method: "deleteMany";
    params: DeleteManyParams;
    resource: T;
    model?: string;
};

declare const auditHandler: (request: RaPayload, options: AuditOptions, created?: any) => Promise<void>;
declare const createAuditEntry: (options: AuditOptions, request: RaPayload, id: any) => Promise<any>;

type CreateArgs = {
    include?: object | null;
    select?: object | null;
};
type CreateOptions<Args extends CreateArgs = CreateArgs> = Args & {
    connect?: {
        [key: string]: string | {
            [key: string]: string | {
                [key: string]: string;
            };
        };
    };
    audit?: AuditOptions;
    debug?: boolean;
};
declare const createHandler: <Args extends CreateArgs>(req: CreateRequest, model: {
    create: Function;
}, options?: CreateOptions<Omit<Args, "data">>) => Promise<{
    data: any;
}>;

type AxiosInterceptorFulfilled<V> = ((value: V) => V | Promise<V>) | null;
type AxiosInterceptorError = ((error: any) => any) | null;
declare const dataProvider: (endpoint: string, options?: {
    headers?: CreateAxiosDefaults["headers"];
    withCredentials?: boolean;
    resourceToModelMap?: Record<string, string>;
    axiosInterceptors?: {
        response?: {
            onFulfilled?: AxiosInterceptorFulfilled<any>;
            onRejected?: AxiosInterceptorError;
            options?: AxiosInterceptorOptions;
        }[];
        request?: {
            onFulfilled?: AxiosInterceptorFulfilled<any>;
            onRejected?: AxiosInterceptorError;
            options?: AxiosInterceptorOptions;
        }[];
    };
}) => DataProvider;

type DeleteOptions = {
    softDeleteField?: string;
    debug?: boolean;
    audit?: AuditOptions;
};
declare const deleteHandler: <W extends {
    include?: object | null;
    select?: object | null;
    where?: object | null;
}>(req: DeleteRequest, model: {
    update: Function;
    delete: Function;
}, options?: DeleteOptions) => Promise<{
    data: any;
}>;

type FilterMode = "insensitive" | "default" | undefined;
type ExtractWhereOptions = {
    filterMode?: FilterMode;
    debug?: boolean;
};
declare const extractWhere: (req: GetListRequest | GetManyReferenceRequest, options?: ExtractWhereOptions) => {};

type GetListArgs = {
    include?: object | null;
    select?: object | null;
    where?: object | null;
    orderBy?: object | null;
};
type GetListOptions<Args extends GetListArgs = GetListArgs> = Args & {
    noNullsOnSort?: string[];
    debug?: boolean;
    transformRow?: (row: any, rowIndex: number, rows: any[]) => any | Promise<any>;
    filterMode?: FilterMode;
};
declare const getListHandler: <Args extends GetListArgs>(req: GetListRequest, model: {
    findMany: Function;
    count: Function;
}, options?: GetListOptions<Args>) => Promise<{
    data: any;
    total: any;
}>;

type UpdateArgs = {
    include?: object | null;
    select?: object | null;
};
type UpdateOptions<Args extends UpdateArgs = UpdateArgs> = Args & {
    debug?: boolean;
    skipFields?: {
        [key: string]: boolean;
    };
    allowOnlyFields?: {
        [key: string]: boolean;
    };
    set?: {
        [key: string]: string | {
            [key: string]: string | {
                [key: string]: string;
            };
        };
    };
    allowNestedUpdate?: {
        [key: string]: boolean;
    };
    allowNestedUpsert?: {
        [key: string]: boolean;
    };
    allowJsonUpdate?: {
        [key: string]: boolean;
    };
    audit?: AuditOptions;
};
declare const reduceData: (data: any, options: UpdateOptions) => {};
declare const updateHandler: <Args extends UpdateArgs>(req: UpdateRequest, model: {
    update: Function;
}, options?: UpdateOptions<Omit<Args, "data" | "where">>) => Promise<{
    data: any;
}>;

type GetManyArgs = {
    include?: object | null;
    select?: object | null;
};
type GetManyOptions<Args extends GetManyArgs = GetManyArgs> = Args & {
    debug?: boolean;
    transformRow?: (data: any) => any | Promise<any>;
};
declare const getManyHandler: <Args extends GetManyArgs>(req: GetManyRequest, model: {
    findMany: Function;
}, options?: GetManyOptions<Args>) => Promise<{
    data: any;
}>;

type GetManyRefernceArgs = {
    include?: object | null;
    select?: object | null;
};
type GetManyReferenceOptions<Args extends GetManyRefernceArgs = GetManyRefernceArgs> = Args & {
    debug?: boolean;
    transformRow?: (data: any) => any | Promise<any>;
    filterMode?: FilterMode;
};
declare const getManyReferenceHandler: <Args extends GetManyRefernceArgs>(req: GetManyReferenceRequest, model: {
    findMany: Function;
}, options?: GetManyReferenceOptions<Args>) => Promise<{
    data: any;
    total: any;
}>;

type GetOneArgs = {
    include?: object | null;
    select?: object | null;
};
type GetOneOptions<Args extends GetOneArgs = GetOneArgs> = Args & {
    debug?: boolean;
    transform?: (row: any) => any | Promise<any>;
};
declare const getOneHandler: <Args extends GetOneArgs>(req: GetOneRequest, model: {
    findUnique: Function;
}, options?: GetOneOptions<Omit<Args, "where">>) => Promise<{
    data: any;
}>;

declare const defaultHandler: (req: RaPayload, prisma: PrismaClient, options?: {
    audit?: AuditOptions;
    create?: CreateOptions;
    delete?: DeleteOptions;
    getList?: GetListOptions;
    getMany?: GetManyOptions;
    getManyReference?: GetManyReferenceOptions;
    getOne?: GetOneOptions;
    update?: UpdateOptions;
}) => Promise<{
    data: any;
}>;

type DeleteManyOptions = {
    softDeleteField?: string;
    audit?: AuditOptions;
    debug?: boolean;
};
declare const deleteManyHandler: (req: DeleteManyRequest, model: {
    updateMany: Function;
    deleteMany: Function;
}, options?: DeleteManyOptions) => Promise<{
    data: any[];
}>;

declare const extractOrderBy: (req: GetListRequest | GetManyReferenceRequest) => {};

declare const extractSkipTake: (req: GetListRequest | GetManyReferenceRequest) => {
    skip: any;
    take: any;
};

declare const getInfiniteListHandler: <Args extends GetListArgs>(req: GetListRequest, model: {
    findMany: Function;
    count: Function;
}, options?: GetListOptions<Args>) => Promise<{
    data: any;
    pageInfo: {
        hasPreviousPage: boolean;
        hasNextPage: boolean;
    };
}>;

declare const isExport: (req: GetListRequest) => boolean;

interface Permission<T> {
    /**
     * @default 'allow'
     */
    type?: "allow" | "deny";
    /**
     * List of actions to allow/deny for the resource
     * Can be a single action or an array of actions
     */
    action: Action | Action[];
    /**
     * Resource to allow/deny the actions on
     * This is ideally typed but is also of type string for joint resources ex. user.email
     */
    resource: T | "*" | string;
    record?: any;
}
type Action = "*" | "list" | "show" | "create" | "edit" | "delete" | "export";
type Permissions<T> = Permission<T>[];

type ReactAdminFetchActions = "getOne" | "create" | "update" | "getList" | "getMany" | "getManyReference" | "updateMany" | "delete" | "deleteMany";
declare const fetchActionToAction: Record<ReactAdminFetchActions, Action>;
declare const canAccess: ({ action, permissions, resource, record, }: {
    action: Action;
    permissions: Permissions<string>;
    resource: string;
    record?: any;
}) => boolean;

declare const updateManyHandler: <Args extends UpdateArgs>(req: UpdateManyRequest, model: {
    updateMany: Function;
}, options?: Omit<UpdateOptions<Args>, "select" | "include">) => Promise<{
    data: ra_core.Identifier[];
}>;

export { type Action, type AuditActions, type AuditOptions, type CreateArgs, type CreateOptions, type CreateRequest, type DeleteManyOptions, type DeleteManyRequest, type DeleteOptions, type DeleteRequest, type ExtractWhereOptions, type FilterMode, type GetListArgs, type GetListOptions, type GetListRequest, type GetManyArgs, type GetManyOptions, type GetManyReferenceOptions, type GetManyReferenceRequest, type GetManyRefernceArgs, type GetManyRequest, type GetOneArgs, type GetOneOptions, type GetOneRequest, type Permission, type Permissions, type RaPayload, type ReactAdminFetchActions, type UpdateArgs, type UpdateManyRequest, type UpdateOptions, type UpdateRequest, auditHandler, canAccess, createAuditEntry, createHandler, dataProvider, defaultAuditOptions, defaultHandler, deleteHandler, deleteManyHandler, extractOrderBy, extractSkipTake, extractWhere, fetchActionToAction, getInfiniteListHandler, getListHandler, getManyHandler, getManyReferenceHandler, getOneHandler, isExport, reduceData, updateHandler, updateManyHandler };
